static const char interfaceXml0[] = R"XML_DELIMITER(<?xml version="1.0" encoding="UTF-8" ?>
<!DOCTYPE node PUBLIC "-//freedesktop//DTD D-BUS Object Introspection 1.0//EN" "http://www.freedesktop.org/standards/dbus/1.0/introspect.dtd">
<node xmlns:doc="http://www.freedesktop.org/dbus/1.0/doc.dtd">
    <interface name="org.gnome.SessionManager.ClientPrivate">
        <method name="EndSessionResponse">
            <arg name="is_ok" type="b" direction="in">
                <summary>Whether or not it is OK to preceed</summary>
            </arg>
            <arg name="reason" type="s" direction="in">
                <summary>The reason string</summary>
            </arg>
            <description>This method should be used by the client in response to the QueryEndSession and EndSession signals.</description>
        </method>

        <signal name="Stop">
            <arg name="placeholder" type="b" />
            <description>The client should stop and remove itself from the session in response to this signal.</description>
        </signal>

        <signal name="QueryEndSession">
            <arg name="flags" type="u">
                <summary>Flags</summary>
            </arg>
            <description>This signal is used to inform the client that the session is about to end.  The client must respond 
            by calling EndSessionResponse within one second of the signal emission. If the client responds with an EndSessionResponse
            is-ok argument equal to FALSE and a reason then this reason may be displayed to the user. The client must not attempt to
            perform any actions or interact with the user in response to this signal.  Any actions required for a clean shutdown should
            take place in response to the EndSession signal. The client should limit operations until either a EndSession or CancelEndSession
            signal is received.</description>
        </signal>

        <signal name="EndSession">
            <arg name="flags" type="u">
                <summary>Flags</summary>
            </arg>
            <description>This signal is used to inform the client that the session is about to end.  The client must respond by
            calling EndSessionResponse within ten seconds of the signal emission. The client must not attempt to interact with
            the user in response to this signal. The application will be given a maxium of ten seconds to perform any actions required
            for a clean shutdown.</description>
        </signal>

        <signal name="CancelEndSession">
            <arg name="placeholder" type="b" />
            <description>This signal indicates to the client that a previous emission ofQueryEndSession has been cancelled.
            The client should resume normal operations.</description>
        </signal>

    </interface>
</node>
)XML_DELIMITER";

#include "/home/abuild/rpmbuild/BUILD/kiran-session-manager-2.1.2/generated/client_private_dbus_stub.h"

template<class T>
inline T specialGetter(Glib::Variant<T> variant)
{
    return variant.get();
}

template<>
inline std::string specialGetter(Glib::Variant<std::string> variant)
{
    // String is not guaranteed to be null-terminated, so don't use ::get()
    gsize n_elem;
    gsize elem_size = sizeof(char);
    char* data = (char*)g_variant_get_fixed_array(variant.gobj(), &n_elem, elem_size);

    return std::string(data, n_elem);
}

gnome::SessionManager::ClientPrivateStub::ClientPrivateStub():
    m_interfaceName("org.gnome.SessionManager.ClientPrivate")
{
    Stop_signal.connect(sigc::bind<0>(sigc::mem_fun(this, &ClientPrivateStub::Stop_emitter),
            std::vector<Glib::ustring>({""})) );
    Stop_selectiveSignal.connect(sigc::mem_fun(this, &ClientPrivateStub::Stop_emitter));
    QueryEndSession_signal.connect(sigc::bind<0>(sigc::mem_fun(this, &ClientPrivateStub::QueryEndSession_emitter),
            std::vector<Glib::ustring>({""})) );
    QueryEndSession_selectiveSignal.connect(sigc::mem_fun(this, &ClientPrivateStub::QueryEndSession_emitter));
    EndSession_signal.connect(sigc::bind<0>(sigc::mem_fun(this, &ClientPrivateStub::EndSession_emitter),
            std::vector<Glib::ustring>({""})) );
    EndSession_selectiveSignal.connect(sigc::mem_fun(this, &ClientPrivateStub::EndSession_emitter));
    CancelEndSession_signal.connect(sigc::bind<0>(sigc::mem_fun(this, &ClientPrivateStub::CancelEndSession_emitter),
            std::vector<Glib::ustring>({""})) );
    CancelEndSession_selectiveSignal.connect(sigc::mem_fun(this, &ClientPrivateStub::CancelEndSession_emitter));
}

gnome::SessionManager::ClientPrivateStub::~ClientPrivateStub()
{
    unregister_object();
}

guint gnome::SessionManager::ClientPrivateStub::register_object(
    const Glib::RefPtr<Gio::DBus::Connection> &connection,
    const Glib::ustring &object_path)
{
    if (!introspection_data) {
        try {
            introspection_data = Gio::DBus::NodeInfo::create_for_xml(interfaceXml0);
        } catch(const Glib::Error& ex) {
            g_warning("Unable to create introspection data for %s: %s", object_path.c_str(), ex.what().c_str());
            return 0;
        }
    }

    Gio::DBus::InterfaceVTable *interface_vtable =
        new Gio::DBus::InterfaceVTable(
            sigc::mem_fun(this, &ClientPrivateStub::on_method_call),
            sigc::mem_fun(this, &ClientPrivateStub::on_interface_get_property),
            sigc::mem_fun(this, &ClientPrivateStub::on_interface_set_property));

    guint registration_id;
    try {
        registration_id = connection->register_object(object_path,
            introspection_data->lookup_interface("org.gnome.SessionManager.ClientPrivate"),
            *interface_vtable);
    } catch(const Glib::Error &ex) {
        g_warning("Registration of object %s failed: %s", object_path.c_str(), ex.what().c_str());
        return 0;
    }

    m_registered_objects.emplace_back(RegisteredObject {
        registration_id,
        connection,
        object_path
    });

    return registration_id;
}

void gnome::SessionManager::ClientPrivateStub::unregister_object()
{
    for (const RegisteredObject &obj: m_registered_objects) {
        obj.connection->unregister_object(obj.id);
    }
    m_registered_objects.clear();
}

void gnome::SessionManager::ClientPrivateStub::on_method_call(
    const Glib::RefPtr<Gio::DBus::Connection> &/* connection */,
    const Glib::ustring &/* sender */,
    const Glib::ustring &/* object_path */,
    const Glib::ustring &/* interface_name */,
    const Glib::ustring &method_name,
    const Glib::VariantContainerBase &parameters,
    const Glib::RefPtr<Gio::DBus::MethodInvocation> &invocation)
{
    static_cast<void>(method_name); // maybe unused
    static_cast<void>(parameters); // maybe unused
    static_cast<void>(invocation); // maybe unused

    if (method_name.compare("EndSessionResponse") == 0) {
        Glib::Variant<bool> base_is_ok;
        parameters.get_child(base_is_ok, 0);
        bool p_is_ok = specialGetter(base_is_ok);

        Glib::Variant<Glib::ustring> base_reason;
        parameters.get_child(base_reason, 1);
        Glib::ustring p_reason = specialGetter(base_reason);

        MethodInvocation methodInvocation(invocation);
        EndSessionResponse(
            (p_is_ok),
            (p_reason),
            methodInvocation);
    }

}

void gnome::SessionManager::ClientPrivateStub::on_interface_get_property(
    Glib::VariantBase &property,
    const Glib::RefPtr<Gio::DBus::Connection> &/* connection */,
    const Glib::ustring &/* sender */,
    const Glib::ustring &/* object_path */,
    const Glib::ustring &/* interface_name */,
    const Glib::ustring &property_name)
{
    static_cast<void>(property); // maybe unused
    static_cast<void>(property_name); // maybe unused

}

bool gnome::SessionManager::ClientPrivateStub::on_interface_set_property(
    const Glib::RefPtr<Gio::DBus::Connection> &/* connection */,
    const Glib::ustring &/* sender */,
    const Glib::ustring &/* object_path */,
    const Glib::ustring &/* interface_name */,
    const Glib::ustring &property_name,
    const Glib::VariantBase &value)
{
    static_cast<void>(property_name); // maybe unused
    static_cast<void>(value); // maybe unused

    return true;
}

void gnome::SessionManager::ClientPrivateStub::Stop_emitter(
    const std::vector<Glib::ustring> &destination_bus_names,bool placeholder)
{
    std::vector<Glib::VariantBase> paramsList;

    paramsList.push_back(Glib::Variant<bool>::create((placeholder)));;

    const Glib::VariantContainerBase params =
        Glib::Variant<std::vector<Glib::VariantBase>>::create_tuple(paramsList);
    for (const RegisteredObject &obj: m_registered_objects) {
        for (const auto &bus_name: destination_bus_names) {
            obj.connection->emit_signal(
                    obj.object_path,
                    "org.gnome.SessionManager.ClientPrivate",
                    "Stop",
                    bus_name,
                    params);
        }
    }
}

void gnome::SessionManager::ClientPrivateStub::QueryEndSession_emitter(
    const std::vector<Glib::ustring> &destination_bus_names,guint32 flags)
{
    std::vector<Glib::VariantBase> paramsList;

    paramsList.push_back(Glib::Variant<guint32>::create((flags)));;

    const Glib::VariantContainerBase params =
        Glib::Variant<std::vector<Glib::VariantBase>>::create_tuple(paramsList);
    for (const RegisteredObject &obj: m_registered_objects) {
        for (const auto &bus_name: destination_bus_names) {
            obj.connection->emit_signal(
                    obj.object_path,
                    "org.gnome.SessionManager.ClientPrivate",
                    "QueryEndSession",
                    bus_name,
                    params);
        }
    }
}

void gnome::SessionManager::ClientPrivateStub::EndSession_emitter(
    const std::vector<Glib::ustring> &destination_bus_names,guint32 flags)
{
    std::vector<Glib::VariantBase> paramsList;

    paramsList.push_back(Glib::Variant<guint32>::create((flags)));;

    const Glib::VariantContainerBase params =
        Glib::Variant<std::vector<Glib::VariantBase>>::create_tuple(paramsList);
    for (const RegisteredObject &obj: m_registered_objects) {
        for (const auto &bus_name: destination_bus_names) {
            obj.connection->emit_signal(
                    obj.object_path,
                    "org.gnome.SessionManager.ClientPrivate",
                    "EndSession",
                    bus_name,
                    params);
        }
    }
}

void gnome::SessionManager::ClientPrivateStub::CancelEndSession_emitter(
    const std::vector<Glib::ustring> &destination_bus_names,bool placeholder)
{
    std::vector<Glib::VariantBase> paramsList;

    paramsList.push_back(Glib::Variant<bool>::create((placeholder)));;

    const Glib::VariantContainerBase params =
        Glib::Variant<std::vector<Glib::VariantBase>>::create_tuple(paramsList);
    for (const RegisteredObject &obj: m_registered_objects) {
        for (const auto &bus_name: destination_bus_names) {
            obj.connection->emit_signal(
                    obj.object_path,
                    "org.gnome.SessionManager.ClientPrivate",
                    "CancelEndSession",
                    bus_name,
                    params);
        }
    }
}


bool gnome::SessionManager::ClientPrivateStub::emitSignal(
    const std::string &propName,
    Glib::VariantBase &value)
{
    std::map<Glib::ustring, Glib::VariantBase> changedProps;
    std::vector<Glib::ustring> changedPropsNoValue;

    changedProps[propName] = value;

    Glib::Variant<std::map<Glib::ustring, Glib::VariantBase>> changedPropsVar =
        Glib::Variant<std::map<Glib::ustring, Glib::VariantBase>>::create(changedProps);
    Glib::Variant<std::vector<Glib::ustring>> changedPropsNoValueVar =
        Glib::Variant<std::vector<Glib::ustring>>::create(changedPropsNoValue);
    std::vector<Glib::VariantBase> ps;
    ps.push_back(Glib::Variant<Glib::ustring>::create(m_interfaceName));
    ps.push_back(changedPropsVar);
    ps.push_back(changedPropsNoValueVar);
    Glib::VariantContainerBase propertiesChangedVariant =
        Glib::Variant<std::vector<Glib::VariantBase>>::create_tuple(ps);

    for (const RegisteredObject &obj: m_registered_objects) {
        obj.connection->emit_signal(
            obj.object_path,
            "org.freedesktop.DBus.Properties",
            "PropertiesChanged",
            Glib::ustring(),
            propertiesChangedVariant);
    }

    return true;
}
