#pragma once
#include <string>
#include <vector>
#include <glibmm.h>
#include <giomm.h>
#include "/home/abuild/rpmbuild/BUILD/kiran-session-manager-2.1.2/generated/presence_dbus_common.h"

namespace gnome {
namespace SessionManager {

class PresenceStub : public sigc::trackable {
public:
    PresenceStub();
    virtual ~PresenceStub();

    PresenceStub(const PresenceStub &other) = delete;
    PresenceStub(PresenceStub &&other) = delete;
    PresenceStub &operator=(const PresenceStub &other) = delete;
    PresenceStub &operator=(PresenceStub &&other) = delete;

    guint register_object(const Glib::RefPtr<Gio::DBus::Connection> &connection,
                          const Glib::ustring &object_path);
    void unregister_object();

    unsigned int usage_count() const {
        return static_cast<unsigned int>(m_registered_objects.size());
    }

    class MethodInvocation;

    bool status_set(guint32 value);
    bool status_text_set(const Glib::ustring & value);
protected:
    virtual void SetStatus(
        guint32 status,
        MethodInvocation &invocation) = 0;
    virtual void SetStatusText(
        const Glib::ustring & status_text,
        MethodInvocation &invocation) = 0;

    /* Handle the setting of a property
     * This method will be called as a result of a call to <PropName>_set
     * and should implement the actual setting of the property value.
     * Should return true on success and false otherwise.
     */
    virtual bool status_setHandler(guint32 value) = 0;
    virtual guint32 status_get() = 0;

    /* Handle the setting of a property
     * This method will be called as a result of a call to <PropName>_set
     * and should implement the actual setting of the property value.
     * Should return true on success and false otherwise.
     */
    virtual bool status_text_setHandler(const Glib::ustring & value) = 0;
    virtual Glib::ustring status_text_get() = 0;

    void StatusChanged_emitter(const std::vector<Glib::ustring> &destination_bus_names, guint32);
    sigc::signal<void, guint32> StatusChanged_signal;
    sigc::signal<void, const std::vector<Glib::ustring>&, guint32> StatusChanged_selectiveSignal;

    void StatusTextChanged_emitter(const std::vector<Glib::ustring> &destination_bus_names, const Glib::ustring &);
    sigc::signal<void, const Glib::ustring &> StatusTextChanged_signal;
    sigc::signal<void, const std::vector<Glib::ustring>&, const Glib::ustring &> StatusTextChanged_selectiveSignal;

    void on_method_call(const Glib::RefPtr<Gio::DBus::Connection> &connection,
                        const Glib::ustring &sender,
                        const Glib::ustring &object_path,
                        const Glib::ustring &interface_name,
                        const Glib::ustring &method_name,
                        const Glib::VariantContainerBase &parameters,
                        const Glib::RefPtr<Gio::DBus::MethodInvocation> &invocation);

    void on_interface_get_property(Glib::VariantBase& property,
                                   const Glib::RefPtr<Gio::DBus::Connection> &connection,
                                   const Glib::ustring &sender,
                                   const Glib::ustring &object_path,
                                   const Glib::ustring &interface_name,
                                   const Glib::ustring &property_name);

    bool on_interface_set_property(
        const Glib::RefPtr<Gio::DBus::Connection> &connection,
        const Glib::ustring &sender,
        const Glib::ustring &object_path,
        const Glib::ustring &interface_name,
        const Glib::ustring &property_name,
        const Glib::VariantBase &value);

private:
    bool emitSignal(const std::string &propName, Glib::VariantBase &value);

    struct RegisteredObject {
        guint id;
        Glib::RefPtr<Gio::DBus::Connection> connection;
        std::string object_path;
    };

    Glib::RefPtr<Gio::DBus::NodeInfo> introspection_data;
    std::vector<RegisteredObject> m_registered_objects;
    std::string m_interfaceName;
};

class PresenceStub::MethodInvocation {
public:
    MethodInvocation(const Glib::RefPtr<Gio::DBus::MethodInvocation> &msg):
        m_message(msg) {}

    const Glib::RefPtr<Gio::DBus::MethodInvocation> getMessage() {
        return m_message;
    }

    void ret(Glib::Error error) {
        m_message->return_error(error);
    }

    void returnError(const Glib::ustring &domain, int code, const Glib::ustring &message) {
        m_message->return_error(domain, code, message);
    }

    void ret() {
        std::vector<Glib::VariantBase> vlist;

        m_message->return_value(Glib::Variant<Glib::VariantBase>::create_tuple(vlist));
    }

private:
    Glib::RefPtr<Gio::DBus::MethodInvocation> m_message;
};

} // SessionManager
} // gnome

