#pragma once
#include <iostream>
#include <vector>
#include "glibmm.h"
#include "giomm.h"

namespace gnome {

class SessionManagerTypeWrap {
public:
    template<typename T>
    static void unwrapList(std::vector<T> &list, const Glib::VariantContainerBase &wrapped) {
        for (uint i = 0; i < wrapped.get_n_children(); i++) {
            Glib::Variant<T> item;
            wrapped.get_child(item, i);
            list.push_back(item.get());
        }
    }

    static std::vector<Glib::ustring> stdStringVecToGlibStringVec(const std::vector<std::string> &strv) {
        std::vector<Glib::ustring> newStrv;
        for (uint i = 0; i < strv.size(); i++) {
            newStrv.push_back(strv[i]);
        }

        return newStrv;
    }

    static std::vector<std::string> glibStringVecToStdStringVec(const std::vector<Glib::ustring> &strv) {
        std::vector<std::string> newStrv;
        for (uint i = 0; i < strv.size(); i++) {
            newStrv.push_back(strv[i]);
        }

        return newStrv;
    }

    static Glib::VariantContainerBase RegisterClient_pack(
        const Glib::ustring & arg_app_id,
        const Glib::ustring & arg_client_startup_id) {
        Glib::VariantContainerBase base;
        std::vector<Glib::VariantBase> params;

        Glib::Variant<Glib::ustring> app_id_param =
            Glib::Variant<Glib::ustring>::create(arg_app_id);
        params.push_back(app_id_param);

        Glib::Variant<Glib::ustring> client_startup_id_param =
            Glib::Variant<Glib::ustring>::create(arg_client_startup_id);
        params.push_back(client_startup_id_param);
        return Glib::VariantContainerBase::create_tuple(params);
    }

    static Glib::VariantContainerBase Inhibit_pack(
        const Glib::ustring & arg_app_id,
        guint32 arg_toplevel_xid,
        const Glib::ustring & arg_reason,
        guint32 arg_flags) {
        Glib::VariantContainerBase base;
        std::vector<Glib::VariantBase> params;

        Glib::Variant<Glib::ustring> app_id_param =
            Glib::Variant<Glib::ustring>::create(arg_app_id);
        params.push_back(app_id_param);

        Glib::Variant<guint32> toplevel_xid_param =
            Glib::Variant<guint32>::create(arg_toplevel_xid);
        params.push_back(toplevel_xid_param);

        Glib::Variant<Glib::ustring> reason_param =
            Glib::Variant<Glib::ustring>::create(arg_reason);
        params.push_back(reason_param);

        Glib::Variant<guint32> flags_param =
            Glib::Variant<guint32>::create(arg_flags);
        params.push_back(flags_param);
        return Glib::VariantContainerBase::create_tuple(params);
    }

    static Glib::VariantContainerBase Uninhibit_pack(
        guint32 arg_inhibit_cookie) {
        Glib::VariantContainerBase base;
        Glib::Variant<guint32> params =
            Glib::Variant<guint32>::create(arg_inhibit_cookie);
        return Glib::VariantContainerBase::create_tuple(params);
    }

    static Glib::VariantContainerBase IsInhibited_pack(
        guint32 arg_flags) {
        Glib::VariantContainerBase base;
        Glib::Variant<guint32> params =
            Glib::Variant<guint32>::create(arg_flags);
        return Glib::VariantContainerBase::create_tuple(params);
    }

    static Glib::VariantContainerBase GetInhibitor_pack(
        guint32 arg_cookie) {
        Glib::VariantContainerBase base;
        Glib::Variant<guint32> params =
            Glib::Variant<guint32>::create(arg_cookie);
        return Glib::VariantContainerBase::create_tuple(params);
    }

    static Glib::VariantContainerBase Logout_pack(
        guint32 arg_mode) {
        Glib::VariantContainerBase base;
        Glib::Variant<guint32> params =
            Glib::Variant<guint32>::create(arg_mode);
        return Glib::VariantContainerBase::create_tuple(params);
    }
};

} // gnome


