static const char interfaceXml0[] = R"XML_DELIMITER(<?xml version="1.0" encoding="UTF-8" ?>
<!DOCTYPE node PUBLIC "-//freedesktop//DTD D-BUS Object Introspection 1.0//EN" "http://www.freedesktop.org/standards/dbus/1.0/introspect.dtd">
<node xmlns:doc="http://www.freedesktop.org/dbus/1.0/doc.dtd">
    <interface name="org.gnome.SessionManager">
        <method name="RegisterClient">
            <annotation name="org.freedesktop.DBus.GLib.Async" value=""/>
            <arg type="s" name="app_id" direction="in">
                <summary>The application id, that commmonly is desktop filename.</summary>
            </arg>
            <arg type="s" name="client_startup_id" direction="in">
                <summary>Client startup id.</summary>
            </arg>
            <arg type="o" name="client_id" direction="out">
                <summary>The object path for the client allocation.</summary>
            </arg>
            <description>Register a client for the caller.</description>
        </method>

        <method name="Inhibit">
            <annotation name="org.freedesktop.DBus.GLib.Async" value=""/>
            <arg type="s" name="app_id" direction="in">
                <summary>The application id.</summary>
            </arg>
            <arg type="u" name="toplevel_xid" direction="in">
                <summary>The argument is ignored.</summary>
            </arg>
            <arg type="s" name="reason" direction="in">
                <summary>The reason for the inhibit.</summary>
            </arg>
            <arg type="u" name="flags" direction="in">
                <summary>Specify what should be inhibited, Refer to KSMInhibitorFlag in ksm-i.h</summary>
            </arg>
            <arg type="u" name="inhibit_cookie" direction="out">
                <summary>The inhibitor cookie that is used to Uninhibit.</summary>
            </arg>

            <description>Inhibit that the session manager quit(logout, shutdown, reboot), save(hibernate, suspend) and idle.</description>
        </method>

        <method name="Uninhibit">
            <annotation name="org.freedesktop.DBus.GLib.Async" value=""/>
            <arg type="u" name="inhibit_cookie" direction="in">
                <summary>The inhibitor cookie.</summary>
            </arg>
            <description>Cancel the inhibitor.</description>
        </method>

        <method name="IsInhibited">
            <arg type="u" name="flags" direction="in">
                <summary>Specify the inhibitor flag, Refer to KSMInhibitorFlag in ksm-i.h</summary>
            </arg>
            <arg type="b" name="is_inhibited" direction="out">
                <summary>Return true if all the bit in flags is set.</summary>
            </arg>
            <description>Determine if the operation specified by flags is set.</description>
        </method>

        <method name="GetInhibitor">
            <arg name="cookie" direction="in" type="u">
                <summary>The inhibitor cookie.</summary>
            </arg>
            <arg name="inhibitor" direction="out" type="s">
                <summary>A json string that contains inhibitor informations.</summary>
            </arg>
            <description>This gets an inhibitor that are currently known to the session manager.</description>
        </method>

        <method name="GetInhibitors">
            <arg name="inhibitors" direction="out" type="s">
                <summary>A json string that contain all inhibitors informations.</summary>
            </arg>
            <description>This gets a list of all the inhibitors that are currently known to the session manager.</description>
        </method>

        <method name="Shutdown">
            <description>Shutdown the system.</description>
        </method>

        <method name="RequestShutdown">
            <description>Shutdown the system, the interface is designed to compatible with other application.</description>
        </method>

        <method name="CanShutdown">
            <arg name="is_available" direction="out" type="b">
                <summary>True if shutdown is available to the user, false otherwise.</summary>
            </arg>
            <description>Whether the user can shutdown.</description>
        </method>

        <method name="Logout">
            <arg name="mode" type="u" direction="in">
                <summary>The argument is ignored.</summary>
            </arg>
            <description>Logout the session.</description>
        </method>

        <method name="CanLogout">
            <arg name="is_available" direction="out" type="b">
                <summary>True if logout is available to the user, false otherwise.</summary>
            </arg>
            <description>Whether the user can logout.</description>
        </method>

        <method name="Reboot">
            <description>Reboot the system.</description>
        </method>

        <method name="RequestReboot">
            <description>Reboot the system, the interface is designed to compatible with other application.</description>
        </method>

        <method name="CanReboot">
            <arg name="is_available" direction="out" type="b">
                <summary>True if reboot is available to the user, false otherwise</summary>
            </arg>
            <description>Whether the user can reboot.</description>
        </method>

        <signal name="InhibitorAdded">
            <arg name="cookie" type="u">
                <summary>The inhibitor cookie.</summary>
            </arg>
            <description>Emitted when an inhibitor has been added to the session manager.</description>
        </signal>

        <signal name="InhibitorRemoved">
            <arg name="cookie" type="u">
                <summary>The inhibitor cookie.</summary>
            </arg>
            <description>Emitted when an inhibitor has been removed from the session manager.</description>
        </signal>

        <signal name="PhaseChanged">
            <arg name="phase" type="i" />
            <description>The stage where session manager is in.</description>
        </signal>

    </interface>
</node>
)XML_DELIMITER";

#include "/home/abuild/rpmbuild/BUILD/kiran-session-manager-2.1.2/generated/session_manager_dbus_stub.h"

template<class T>
inline T specialGetter(Glib::Variant<T> variant)
{
    return variant.get();
}

template<>
inline std::string specialGetter(Glib::Variant<std::string> variant)
{
    // String is not guaranteed to be null-terminated, so don't use ::get()
    gsize n_elem;
    gsize elem_size = sizeof(char);
    char* data = (char*)g_variant_get_fixed_array(variant.gobj(), &n_elem, elem_size);

    return std::string(data, n_elem);
}

gnome::SessionManagerStub::SessionManagerStub():
    m_interfaceName("org.gnome.SessionManager")
{
    InhibitorAdded_signal.connect(sigc::bind<0>(sigc::mem_fun(this, &SessionManagerStub::InhibitorAdded_emitter),
            std::vector<Glib::ustring>({""})) );
    InhibitorAdded_selectiveSignal.connect(sigc::mem_fun(this, &SessionManagerStub::InhibitorAdded_emitter));
    InhibitorRemoved_signal.connect(sigc::bind<0>(sigc::mem_fun(this, &SessionManagerStub::InhibitorRemoved_emitter),
            std::vector<Glib::ustring>({""})) );
    InhibitorRemoved_selectiveSignal.connect(sigc::mem_fun(this, &SessionManagerStub::InhibitorRemoved_emitter));
    PhaseChanged_signal.connect(sigc::bind<0>(sigc::mem_fun(this, &SessionManagerStub::PhaseChanged_emitter),
            std::vector<Glib::ustring>({""})) );
    PhaseChanged_selectiveSignal.connect(sigc::mem_fun(this, &SessionManagerStub::PhaseChanged_emitter));
}

gnome::SessionManagerStub::~SessionManagerStub()
{
    unregister_object();
}

guint gnome::SessionManagerStub::register_object(
    const Glib::RefPtr<Gio::DBus::Connection> &connection,
    const Glib::ustring &object_path)
{
    if (!introspection_data) {
        try {
            introspection_data = Gio::DBus::NodeInfo::create_for_xml(interfaceXml0);
        } catch(const Glib::Error& ex) {
            g_warning("Unable to create introspection data for %s: %s", object_path.c_str(), ex.what().c_str());
            return 0;
        }
    }

    Gio::DBus::InterfaceVTable *interface_vtable =
        new Gio::DBus::InterfaceVTable(
            sigc::mem_fun(this, &SessionManagerStub::on_method_call),
            sigc::mem_fun(this, &SessionManagerStub::on_interface_get_property),
            sigc::mem_fun(this, &SessionManagerStub::on_interface_set_property));

    guint registration_id;
    try {
        registration_id = connection->register_object(object_path,
            introspection_data->lookup_interface("org.gnome.SessionManager"),
            *interface_vtable);
    } catch(const Glib::Error &ex) {
        g_warning("Registration of object %s failed: %s", object_path.c_str(), ex.what().c_str());
        return 0;
    }

    m_registered_objects.emplace_back(RegisteredObject {
        registration_id,
        connection,
        object_path
    });

    return registration_id;
}

void gnome::SessionManagerStub::unregister_object()
{
    for (const RegisteredObject &obj: m_registered_objects) {
        obj.connection->unregister_object(obj.id);
    }
    m_registered_objects.clear();
}

void gnome::SessionManagerStub::on_method_call(
    const Glib::RefPtr<Gio::DBus::Connection> &/* connection */,
    const Glib::ustring &/* sender */,
    const Glib::ustring &/* object_path */,
    const Glib::ustring &/* interface_name */,
    const Glib::ustring &method_name,
    const Glib::VariantContainerBase &parameters,
    const Glib::RefPtr<Gio::DBus::MethodInvocation> &invocation)
{
    static_cast<void>(method_name); // maybe unused
    static_cast<void>(parameters); // maybe unused
    static_cast<void>(invocation); // maybe unused

    if (method_name.compare("RegisterClient") == 0) {
        Glib::Variant<Glib::ustring> base_app_id;
        parameters.get_child(base_app_id, 0);
        Glib::ustring p_app_id = specialGetter(base_app_id);

        Glib::Variant<Glib::ustring> base_client_startup_id;
        parameters.get_child(base_client_startup_id, 1);
        Glib::ustring p_client_startup_id = specialGetter(base_client_startup_id);

        MethodInvocation methodInvocation(invocation);
        RegisterClient(
            (p_app_id),
            (p_client_startup_id),
            methodInvocation);
    }

    if (method_name.compare("Inhibit") == 0) {
        Glib::Variant<Glib::ustring> base_app_id;
        parameters.get_child(base_app_id, 0);
        Glib::ustring p_app_id = specialGetter(base_app_id);

        Glib::Variant<guint32> base_toplevel_xid;
        parameters.get_child(base_toplevel_xid, 1);
        guint32 p_toplevel_xid = specialGetter(base_toplevel_xid);

        Glib::Variant<Glib::ustring> base_reason;
        parameters.get_child(base_reason, 2);
        Glib::ustring p_reason = specialGetter(base_reason);

        Glib::Variant<guint32> base_flags;
        parameters.get_child(base_flags, 3);
        guint32 p_flags = specialGetter(base_flags);

        MethodInvocation methodInvocation(invocation);
        Inhibit(
            (p_app_id),
            (p_toplevel_xid),
            (p_reason),
            (p_flags),
            methodInvocation);
    }

    if (method_name.compare("Uninhibit") == 0) {
        Glib::Variant<guint32> base_inhibit_cookie;
        parameters.get_child(base_inhibit_cookie, 0);
        guint32 p_inhibit_cookie = specialGetter(base_inhibit_cookie);

        MethodInvocation methodInvocation(invocation);
        Uninhibit(
            (p_inhibit_cookie),
            methodInvocation);
    }

    if (method_name.compare("IsInhibited") == 0) {
        Glib::Variant<guint32> base_flags;
        parameters.get_child(base_flags, 0);
        guint32 p_flags = specialGetter(base_flags);

        MethodInvocation methodInvocation(invocation);
        IsInhibited(
            (p_flags),
            methodInvocation);
    }

    if (method_name.compare("GetInhibitor") == 0) {
        Glib::Variant<guint32> base_cookie;
        parameters.get_child(base_cookie, 0);
        guint32 p_cookie = specialGetter(base_cookie);

        MethodInvocation methodInvocation(invocation);
        GetInhibitor(
            (p_cookie),
            methodInvocation);
    }

    if (method_name.compare("GetInhibitors") == 0) {
        MethodInvocation methodInvocation(invocation);
        GetInhibitors(
            methodInvocation);
    }

    if (method_name.compare("Shutdown") == 0) {
        MethodInvocation methodInvocation(invocation);
        Shutdown(
            methodInvocation);
    }

    if (method_name.compare("RequestShutdown") == 0) {
        MethodInvocation methodInvocation(invocation);
        RequestShutdown(
            methodInvocation);
    }

    if (method_name.compare("CanShutdown") == 0) {
        MethodInvocation methodInvocation(invocation);
        CanShutdown(
            methodInvocation);
    }

    if (method_name.compare("Logout") == 0) {
        Glib::Variant<guint32> base_mode;
        parameters.get_child(base_mode, 0);
        guint32 p_mode = specialGetter(base_mode);

        MethodInvocation methodInvocation(invocation);
        Logout(
            (p_mode),
            methodInvocation);
    }

    if (method_name.compare("CanLogout") == 0) {
        MethodInvocation methodInvocation(invocation);
        CanLogout(
            methodInvocation);
    }

    if (method_name.compare("Reboot") == 0) {
        MethodInvocation methodInvocation(invocation);
        Reboot(
            methodInvocation);
    }

    if (method_name.compare("RequestReboot") == 0) {
        MethodInvocation methodInvocation(invocation);
        RequestReboot(
            methodInvocation);
    }

    if (method_name.compare("CanReboot") == 0) {
        MethodInvocation methodInvocation(invocation);
        CanReboot(
            methodInvocation);
    }

}

void gnome::SessionManagerStub::on_interface_get_property(
    Glib::VariantBase &property,
    const Glib::RefPtr<Gio::DBus::Connection> &/* connection */,
    const Glib::ustring &/* sender */,
    const Glib::ustring &/* object_path */,
    const Glib::ustring &/* interface_name */,
    const Glib::ustring &property_name)
{
    static_cast<void>(property); // maybe unused
    static_cast<void>(property_name); // maybe unused

}

bool gnome::SessionManagerStub::on_interface_set_property(
    const Glib::RefPtr<Gio::DBus::Connection> &/* connection */,
    const Glib::ustring &/* sender */,
    const Glib::ustring &/* object_path */,
    const Glib::ustring &/* interface_name */,
    const Glib::ustring &property_name,
    const Glib::VariantBase &value)
{
    static_cast<void>(property_name); // maybe unused
    static_cast<void>(value); // maybe unused

    return true;
}

void gnome::SessionManagerStub::InhibitorAdded_emitter(
    const std::vector<Glib::ustring> &destination_bus_names,guint32 cookie)
{
    std::vector<Glib::VariantBase> paramsList;

    paramsList.push_back(Glib::Variant<guint32>::create((cookie)));;

    const Glib::VariantContainerBase params =
        Glib::Variant<std::vector<Glib::VariantBase>>::create_tuple(paramsList);
    for (const RegisteredObject &obj: m_registered_objects) {
        for (const auto &bus_name: destination_bus_names) {
            obj.connection->emit_signal(
                    obj.object_path,
                    "org.gnome.SessionManager",
                    "InhibitorAdded",
                    bus_name,
                    params);
        }
    }
}

void gnome::SessionManagerStub::InhibitorRemoved_emitter(
    const std::vector<Glib::ustring> &destination_bus_names,guint32 cookie)
{
    std::vector<Glib::VariantBase> paramsList;

    paramsList.push_back(Glib::Variant<guint32>::create((cookie)));;

    const Glib::VariantContainerBase params =
        Glib::Variant<std::vector<Glib::VariantBase>>::create_tuple(paramsList);
    for (const RegisteredObject &obj: m_registered_objects) {
        for (const auto &bus_name: destination_bus_names) {
            obj.connection->emit_signal(
                    obj.object_path,
                    "org.gnome.SessionManager",
                    "InhibitorRemoved",
                    bus_name,
                    params);
        }
    }
}

void gnome::SessionManagerStub::PhaseChanged_emitter(
    const std::vector<Glib::ustring> &destination_bus_names,gint32 phase)
{
    std::vector<Glib::VariantBase> paramsList;

    paramsList.push_back(Glib::Variant<gint32>::create((phase)));;

    const Glib::VariantContainerBase params =
        Glib::Variant<std::vector<Glib::VariantBase>>::create_tuple(paramsList);
    for (const RegisteredObject &obj: m_registered_objects) {
        for (const auto &bus_name: destination_bus_names) {
            obj.connection->emit_signal(
                    obj.object_path,
                    "org.gnome.SessionManager",
                    "PhaseChanged",
                    bus_name,
                    params);
        }
    }
}


bool gnome::SessionManagerStub::emitSignal(
    const std::string &propName,
    Glib::VariantBase &value)
{
    std::map<Glib::ustring, Glib::VariantBase> changedProps;
    std::vector<Glib::ustring> changedPropsNoValue;

    changedProps[propName] = value;

    Glib::Variant<std::map<Glib::ustring, Glib::VariantBase>> changedPropsVar =
        Glib::Variant<std::map<Glib::ustring, Glib::VariantBase>>::create(changedProps);
    Glib::Variant<std::vector<Glib::ustring>> changedPropsNoValueVar =
        Glib::Variant<std::vector<Glib::ustring>>::create(changedPropsNoValue);
    std::vector<Glib::VariantBase> ps;
    ps.push_back(Glib::Variant<Glib::ustring>::create(m_interfaceName));
    ps.push_back(changedPropsVar);
    ps.push_back(changedPropsNoValueVar);
    Glib::VariantContainerBase propertiesChangedVariant =
        Glib::Variant<std::vector<Glib::VariantBase>>::create_tuple(ps);

    for (const RegisteredObject &obj: m_registered_objects) {
        obj.connection->emit_signal(
            obj.object_path,
            "org.freedesktop.DBus.Properties",
            "PropertiesChanged",
            Glib::ustring(),
            propertiesChangedVariant);
    }

    return true;
}
